/*****************************************************************************

  3D Randomized Motion Planner

  COPYRIGHT (c) 1990 
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made 
  without the express permission of the author. Any such copies, or 
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author: Jerome Barraquand
  Modified by Tsai-Yen Li (li@flamingo.Stanford.EDU)
  Advisor: Jean-Claude Latombe

******************************************************************************/

#ifndef MIN
#define MIN(A,B) ((A)>(B)?(B):(A))
#endif

#ifndef MAX
#define MAX(A,B) ((A)>(B)?(A):(B))
#endif

#define SQR(X) ((X)*(X))
#define NL(fd) fprintf(fd,"\n")


/*****************************************************************************
  create a node and assign to the head 
******************************************************************************/

#define CONS(type,item,head) \
\
  item=(type *)calloc(1,sizeof(type)); \
  item->next=head; \
  head=item; \
\

/*****************************************************************************
  Check if a point is inside a box 
******************************************************************************/

#define INOBJ(p,dim) (fabs(p.x)  <= 0.5*dim.x && \
		      fabs(p.y)  <= 0.5*dim.y && \
		      fabs(p.z)  <= 0.5*dim.z)

/*****************************************************************************
  test if the point is inside work space 
******************************************************************************/

#define  outside(X,result) {\
  int outside_i;\
  result = 0;\
  for(outside_i = 0;outside_i < therobot.nbpoint;++outside_i) {\
    if(X[outside_i].x <= 0 || X[outside_i].x >= 1 \
       || X[outside_i].y <=0 || X[outside_i].y>= 1 \
       || X[outside_i].z <=0 || X[outside_i].z >= 1) {\
      result = 1;\
      break;\
    }\
  }\
}

/****************************************************************************
  assign a configuration to the other 
******************************************************************************/
  
#define assign(a,b) bcopy(a,b,sizeof(configuration))

/*****************************************************************************
  assign a position to the other 
******************************************************************************/
  
#define assign_pos(a,b)\
  b.x=a.x;\
  b.y=a.y;\
  b.z=a.z;

/*****************************************************************************
  add two configurations
******************************************************************************/

int add_i;

#define add(a,b,c)\
  for(add_i = 0;add_i<therobot.dof;++add_i) c[add_i] = a[add_i] + b[add_i];

/* assign the leading points to the other */

/*****************************************************************************
  assign one configuration to the other
******************************************************************************/

int d_i;

#define assign_lead(leadson,leadcur) {\
  int d_i;\
  for(d_i = 0;d_i <therobot.nblead;++d_i) {\
     leadcur[d_i].x = leadson[d_i].x;\
     leadcur[d_i].y = leadson[d_i].y;\
     leadcur[d_i].z = leadson[d_i].z;\
  }\
}
  
/*****************************************************************************
  compute potential functions 
******************************************************************************/

int Umax,Umin;

/*****************************************************************************
  potential method one: addition of leading points
******************************************************************************/

#define potential1(lead,heurpot,U) {\
  int i;				      \
  for(i=0,U=0;i<therobot.nblead;i++) \
    U +=    heurpot[i].d[lead[i].x][lead[i].y][lead[i].z];\
}

/*****************************************************************************
  potential method one: coef'ed max and min
******************************************************************************/

#define potential2(lead,heurpot,U)\
  \
  Umax =    MAX(heurpot[0].d[lead[0].x][lead[0].y][lead[0].z],heurpot[1].d[lead[1].x][lead[1].y][lead[1].z]);\
  Umin =    MIN(heurpot[0].d[lead[0].x][lead[0].y][lead[0].z],heurpot[1].d[lead[1].x][lead[1].y][lead[1].z]);\
  U=0.9*Umin+0.1*Umax;


/*****************************************************************************
  potential method one: max
******************************************************************************/

#define potential3(lead,heurpot,U)\
  \
  U =    MAX(heurpot[0].d[lead[0].x][lead[0].y][lead[0].z],heurpot[1].d[lead[1].x][lead[1].y][lead[1].z]);\


/* interpolate between two points(configuration) */

/*****************************************************************************
  Interpolate the configuration from start to end by certain intervals
******************************************************************************/

#define INTERPOLATE(start_pt,end_pt,i,intervals,q) \
{ \
    int j; \
    for (j=0; j<therobot.dof; j++) \
      q[j]=(((intervals-i)*start_pt[j]) + (i*end_pt[j]))/intervals; \
}

/*****************************************************************************
 compute the difference between two configurations 
******************************************************************************/

#define Diff(a,b,diff_val)\
  int diff_i;\
  diff_val=0;\
  for (diff_i=0;diff_i<DEGREE;diff_i++) \
     diff_val += fabs(a[diff_i]-b[diff_i]);

/*****************************************************************************
  compute the combination of two 3-D vectors 
******************************************************************************/

#define add_coef_vec(a,v1,b,v2,v3) \
  v3.x=a*v1.x+b*v2.x;\
  v3.y=a*v1.y+b*v2.y;\
  v3.z=a*v1.z+b*v2.z;

/*****************************************************************************
  compute the addition of two 3-D vectors 
******************************************************************************/

#define add_vec(v1,v2,v3) \
  v3.x=v1.x+v2.x;\
  v3.y=v1.y+v2.y;\
  v3.z=v1.z+v2.z;

/*****************************************************************************
  compute the difference of two 3-D vectors 
******************************************************************************/

#define subst_vec(v1,v2,v3) \
  v3.x=v1.x-v2.x;\
  v3.y=v1.y-v2.y;\
  v3.z=v1.z-v2.z;

/*****************************************************************************
 compute the dot_product of two 3-D vectors 
******************************************************************************/

#define dot_prod(p1,p2) ((p1.x)*(p2.x)+(p1.y)*(p2.y)+(p1.z)*(p2.z))

/****************************************************************************
 compute the distance of two 3D points 
******************************************************************************/

#define Pdist(P,Q) sqrt(SQR(P.x-Q.x)+SQR(P.y-Q.y)+SQR(P.z-Q.z))


