/*****************************************************************************

  3D Randomized Motion Planner

  COPYRIGHT (c) 1990 
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made 
  without the express permission of the author. Any such copies, or 
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author: Jerome Barraquand
  Modified by Tsai-Yen Li (li@flamingo.Stanford.EDU)
  Advisor: Jean-Claude Latombe

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <malloc.h>
#include "defs.h"
#include "solidstruct.h"

/*****************************************************************************
  Allocate memory for 2 dimensional char arrays
******************************************************************************/

char **calloc2D(int dy,int dz,int oc){
  char *bv,**v,*p_bv,**p_v;
  
  if((bv=( char *)calloc(dz*dy,sizeof(char)*oc))==NULL) {
    perror("Memory allocation error");
    exit(-1);
  }
  if((v=( char **)calloc(dy,sizeof(char *)))==NULL) {
    perror("Memory allocation error");
    exit(-1);
  }
  
  p_bv=bv;
  p_v=v;
  
  for(p_bv=bv;p_bv-bv<dz*dy*oc;p_bv+=dz*oc)    
    *p_v++ = p_bv;
  
  return(v);
}

/*****************************************************************************
  Allocate memory for 3 dimensional char arrays
******************************************************************************/

char ***calloc3D(int dimx,int dimy,int dimz,int oc) {
  char *bbu,**bu,***u,*p_bbu,**p_bu;
  int x,y;
  
  if((bbu=(char *)calloc(dimz*dimy*dimx*oc,sizeof(char)))==NULL) {
    perror("Memory allocation error");
    exit(-1);
  }
  if((bu=(char **)calloc(dimy*dimx,sizeof(char *)))==NULL) {
    perror("Memory allocation error");
    exit(-1);
  }
  if((u=(char ***)calloc(dimx,sizeof(char **)))==NULL) {
    perror("Memory allocation error");
    exit(-1);
  }
  
  p_bbu = bbu;
  p_bu=bu;
  
  for(x=0;x<dimx;++x) {
    u[x] = p_bu;
    p_bu += dimy;
    for(y=0;y<dimy;++y) {
      u[x][y] = p_bbu;
      p_bbu+=dimz*oc;
    }
  }
  return(u);
}

/*****************************************************************************
  Allocate memory for 3 dimensional float arrays
******************************************************************************/

fpoint3D ***callocf3D(int dimx,int dimy,int dimz,int oc) {
  fpoint3D *bbu,**bu,***u,*p_bbu,**p_bu;
  int x,y;
  
  if((bbu=(fpoint3D *)calloc(dimz*dimy*dimx*oc,sizeof(fpoint3D)))==NULL) {
    perror("Memory allocation error");
    exit(-1);
  }
  if((bu=(fpoint3D **)calloc(dimy*dimx,sizeof(fpoint3D *)))==NULL) {
    perror("Memory allocation error");
    exit(-1);
  }
  if((u=(fpoint3D ***)calloc(dimx,sizeof(fpoint3D **)))==NULL) {
    perror("Memory allocation error");
    exit(-1);
  }
  
  p_bbu = bbu;
  p_bu=bu;
  
  for(x=0;x<dimx;++x) {
    u[x] = p_bu;
    p_bu += dimy;
    for(y=0;y<dimy;++y) {
      u[x][y] = p_bbu;
      p_bbu+=dimz*oc;
    }
  }
  return(u);
}

/*****************************************************************************
  Free memory for 3 dimensional char arrays
******************************************************************************/

void free3D(unsigned char ***bm){
  free(bm[0][0]);
  free(bm[0]);
  free(bm);
}

