/*****************************************************************************

  3D Randomized Motion Planner

  COPYRIGHT (c) 1990 
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made 
  without the express permission of the author. Any such copies, or 
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author: Jerome Barraquand
  Modified by Tsai-Yen Li (li@flamingo.Stanford.EDU)
  Advisor: Jean-Claude Latombe

******************************************************************************/

#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <stdio.h>
#include <math.h>
#include "defs.h"
#include "solidstruct.h"
#include "macro.h"
#include "xsolidp.h"

void perspective(segment *seg,segment2D *seg2d,int nbseg,
		 float f,float d,float theta,float phi,float ty,float tz) {

  int i;
  register double x,y,z;
  double x1,y1,z1,D,lambda,Jx,Jy,Jz,Kx,Ky;
  float vx,vy,vz,ux,uy,uz;
  
  ux = -cos(theta)*cos(phi);
  uy = -sin(theta)*cos(phi);
  uz = -sin(phi);
  
  vx = 0.5 - d * ux;
  vy = 0.5 - d * uy;
  vz = 0.5 - d * uz;
    
  D = sqrt(ux*ux + uy*uy);
  
  Jx = ux*uz/D;
  Jy = uy*uz/D;
  Jz = -D;
  
  Kx = -uy/D;
  Ky = ux/D;
  
  for(i=0;i<nbseg;++i){
    x = seg[i].a.x;
    y = seg[i].a.y;
    z = seg[i].a.z;
    
    lambda = f/ ((x-vx)*ux + (y-vy)*uy + (z-vz)*uz) ;
    
    x1 = lambda* (x-vx);
    y1 = lambda* (y-vy);
    z1 = lambda* (z-vz);
    
    seg2d[i].a.x = 0.5 + (x1 * Jx + y1 * Jy + z1 * Jz)/ty;
    seg2d[i].a.y = 0.5 + (x1 * Kx + y1 * Ky)/tz;

    x = seg[i].b.x;
    y = seg[i].b.y;
    z = seg[i].b.z;
    
    lambda = f/ ((x-vx)*ux + (y-vy)*uy + (z-vz)*uz) ;
    
    x1 = lambda* (x-vx);
    y1 = lambda* (y-vy);
    z1 = lambda* (z-vz);
    
    seg2d[i].b.x = 0.5 + (x1 * Jx + y1 * Jy + z1 * Jz)/ty;
    seg2d[i].b.y = 0.5 + (x1 * Kx + y1 * Ky)/tz;
  }
}/* END */












