/*****************************************************************************

  3D Randomized Motion Planner

  COPYRIGHT (c) 1990 
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made 
  without the express permission of the author. Any such copies, or 
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author: Jerome Barraquand
  Modified by Tsai-Yen Li (li@flamingo.Stanford.EDU)
  Advisor: Jean-Claude Latombe

******************************************************************************/

typedef int iconfiguration[DEGREE]; 
typedef float configuration[DEGREE]; 
typedef int boolean;
typedef enum _ntype {root,mid,prim} ntype;
typedef enum _ptype {cube,cylinder,sphere} ptype;
typedef enum _operation {uni,diff,subs} operation;

typedef struct _path{
  int dim;
  configuration *d;
} path;

typedef struct _ipath{
  int dim;
  iconfiguration *d;
} ipath;

typedef struct _image{
  int dimx;
  int dimy;
  int dimz;
  unsigned char ***d;
} image;

typedef struct _ipoint3D {
  int x;
  int y;
  int z;
} ipoint3D,ivector3D;

typedef struct _ipoint2D {
  int x;
  int y;
} ipoint2D,ivector2D;

typedef struct _fpoint3D {
  float x;
  float y;
  float z;
} fpoint3D,fvector3D;

typedef struct _fpoint2D {
  float x;
  float y;
} fpoint2D,fvector2D;

typedef struct _dpoint3D {
  double x;
  double y;
  double z;
} dpoint3D,dvector3D;

typedef struct _dpoint2D {
  double x;
  double y;
} dpoint2D,dvector2D;

typedef struct _bbox {
  fpoint3D max;
  fpoint3D min;
} bbox;

typedef struct _ibbox {
  ipoint3D max;
  ipoint3D min;
} ibbox;

typedef struct _primitive {
  int objtype;
  int optype;
  float orig[6];
  dvector3D dim;
  bbox box;
  ibbox ibox;
} primitive;

typedef struct _object {
  int nbprim;
  primitive *prim;
  bbox box;
  ibbox ibox;
} object;

typedef struct _csgnode {
  ntype nodetype;
  configuration T;
  struct _csgnode *left;
  struct _csgnode *right;
  operation bop;
  ptype primtype;
  dvector3D dim;
} csgnode;

typedef struct _segment {
  fpoint3D a;
  fpoint3D b;
} segment;

typedef struct _segment2D{
    fpoint2D a;
    fpoint2D b;
} segment2D;

typedef struct _pixel {
    unsigned char x;
    unsigned char y;
    unsigned char z;
} pixel;

typedef struct _descript {
        int octet;
	int dimx;
	int dimy;
	int dimz;
} descript;

typedef struct _point_item {
  ipoint3D point;
  short edge;
  struct _point_item *next;
} point_item;

typedef struct _region_struct{
  int npoints;
  ipoint3D *point_array;
} region_struct;

typedef struct _point_list_item {
  region_struct r_struct;
  struct _point_list_item *next;
} point_list_item;

typedef struct _myPoint{
  ipoint3D point;
  unsigned long region;
} myPoint;

typedef struct _region_node {
  myPoint my_point;
  struct _region_node *next;
} region_node ;

typedef struct _config_node {
  configuration q;
  struct _config_node *next;
} config_node;


typedef struct _rlink {
  double dlength;
  int ilength;
  int endp1;
  int endp2;
} rlink;

typedef struct _refpoint {
  int refframe;
  fpoint3D localp;
} refpoint;

typedef struct _frame {
  int refframe;
  fpoint3D lorig;
  fpoint3D lorie;
  fpoint3D gorig;
  fpoint3D unit[3];
  float maxlimit[6];
  float minlimit[6];
  int dofmask;
} frame;

typedef struct _robot {
  int dof;
  int *rot;
  int *dimc;
  double *coefc;
  int nbpoint;
  int nblink;
  rlink *robotlink;
  int nblead;
  int nbframe;
  frame *localframe;
  refpoint *robotpoint;
  refpoint *leadpoint;
  configuration initq;
  configuration goalq;
  fpoint3D *goallead;
} robot;

